﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Locomotive
{
    public partial class FormLocomotiveConfig : Form
    {
        DrawningLocomotive _locomotive = null;

        private event Action<DrawningLocomotive> eventAddLocomotive;

        public void AddEvent(Action<DrawningLocomotive> ev)
        {
            if (eventAddLocomotive == null)
            {
                eventAddLocomotive = new Action<DrawningLocomotive>(ev);
            }
            else
            {
                eventAddLocomotive += ev;
            }
        }
        private void buttonAdd_Click(object sender, EventArgs e)
        {
            eventAddLocomotive?.Invoke(_locomotive);
            Close();
        }

        public FormLocomotiveConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;

            buttonCancel.Click += (object sender, EventArgs e) => Close();
        }

        private void labelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void panelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void panelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _locomotive = new DrawningLocomotive((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White);
                    break;
                case "labelModifiedObject":
                    _locomotive = new DrawningWarmlyLocomotive((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, 
                                                                Color.White, Color.Black,
                                                                checkBoxPipe.Checked, checkBoxFuelStorage.Checked);
                    break;
            }
            DrawLocomotive();
        }

        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Control).DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void DrawLocomotive()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _locomotive?.SetPosition(5, 5, pictureBoxObject.Width, pictureBoxObject.Height);
            _locomotive?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }

        private void labelBaseColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void labelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            _locomotive.SetBaseColor((Color)e.Data.GetData(typeof(Color)));
            DrawLocomotive();

        }
        private void labelDopColor_DragEnter(object sender, DragEventArgs e)
        {

            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void labelDopColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_locomotive is DrawningWarmlyLocomotive)
            {
                var locomotive = _locomotive as DrawningWarmlyLocomotive;
                locomotive.SetExtraColor((Color)e.Data.GetData(typeof(Color)));
            }
            DrawLocomotive();

        }

    }
}
