﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Locomotive
{
    internal class MapWithSetLocomotivesGeneric <T, U>
        where T : class, IDrawningObject
        where U : AbstractMap
    {
        /// Ширина окна отрисовки
        private readonly int _pictureWidth;
        /// Высота окна отрисовки
        private readonly int _pictureHeight;
        /// Размер занимаемого объектом места (ширина)
        private readonly int _placeSizeWidth = 210;
        /// Размер занимаемого объектом места (высота)
        private readonly int _placeSizeHeight = 90;
        /// Набор объектов
        private readonly SetLocomotivesGeneric<T> _setLocomotives;
        /// Карта
        private readonly U _map;
        /// Конструктор
        public MapWithSetLocomotivesGeneric(int picWidth, int picHeight, U map)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _setLocomotives = new SetLocomotivesGeneric<T>(width * height);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }
        /// Перегрузка оператора сложения
        public static int operator +(MapWithSetLocomotivesGeneric<T, U> map, T locomotive)
        {
            return map._setLocomotives.Insert(locomotive);
        }
        /// Перегрузка оператора вычитания
        public static T operator -(MapWithSetLocomotivesGeneric<T, U> map, int position)
        {
            return map._setLocomotives.Remove(position);
        }
        /// Вывод всего набора объектов
        public Bitmap ShowSet()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawLocomotives(gr);
            return bmp;
        }
        /// Просмотр объекта на карте
        public Bitmap ShowOnMap()
        {
            Shaking();
            foreach (var locomotive in _setLocomotives.GetLocomotives())
            {
                return _map.CreateMap(_pictureWidth, _pictureHeight, locomotive);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// Перемещение объекта по крате
        public Bitmap MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// "Взбалтываем" набор, чтобы все элементы оказались в начале
        private void Shaking()
        {
            int j = _setLocomotives.Count - 1;
            for (int i = 0; i < _setLocomotives.Count; i++)
            {
                if (_setLocomotives[i] == null)
                {
                    for (; j > i; j--)
                    {
                        var locomotive = _setLocomotives[j];
                        if (locomotive != null)
                        {
                            _setLocomotives.Insert(locomotive, i);
                            _setLocomotives.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }
        /// Метод отрисовки фона
        private void DrawBackground(Graphics g)
        {
            Pen pen;

            for (int j = _placeSizeHeight; j < _pictureHeight; j+= _placeSizeHeight)
            {
                //нижняя линия рельс
                pen = new(Color.Black, 5);

                g.DrawLine(pen, 0, j, _pictureWidth, j);
                for (int i = 0; i < _pictureWidth; i+=20)
                {
                    g.DrawLine(pen, i, j, i, j + 10);
                }
                g.DrawLine(pen, 0, j + 10, _pictureWidth, j + 10);

                //верхняя линия рельс
                
                pen = new(Color.DarkGray, 4);

                g.DrawLine(pen, 0, j - 20, _pictureWidth, j - 20);
                for (int i = 0; i < _pictureWidth; i += 20)
                {
                    g.DrawLine(pen, i, j - 20, i, j - 10);
                }
                g.DrawLine(pen, 0, j - 10, _pictureWidth, j - 10);

                //фонари
                for (int i = _placeSizeWidth; i < _pictureWidth; i += _placeSizeWidth)
                {
                    pen = new(Color.Black, 10);
                    g.DrawLine(pen, i, j - _placeSizeHeight + 20, i, j);
                    pen = new(Color.Yellow, 20);
                    g.DrawLine(pen, i, j - _placeSizeHeight + 18, i, j - _placeSizeHeight + 38);
                }
            }
        }


        /// Метод прорисовки объектов
        private void DrawLocomotives(Graphics g)
        {
            int width = _pictureWidth / _placeSizeWidth;
            int height = _pictureHeight / _placeSizeHeight;

            int curWidth = 0;
            int curHeight = 0;

            foreach (var locomotive in _setLocomotives.GetLocomotives())
            {
                // установка позиции
                locomotive?.SetObject(curWidth * _placeSizeWidth + 10, curHeight * _placeSizeHeight + 15, _pictureWidth, _pictureHeight);
                locomotive?.DrawningObject(g);
                if (curWidth < width) curWidth++;
                else
                {
                    curWidth = 0;
                    curHeight++;
                }
            }
        }

        /// Получение данных в виде строки
        public string GetData(char separatorType, char separatorData)
        {
            string data = $"{_map.GetType().Name}{separatorType}";
            foreach (var locomotive in _setLocomotives.GetLocomotives())
            {
                data += $"{locomotive.getInfo()}{separatorData}";
            }
            return data;
        }

        /// Загрузка списка из массива строк
        public void LoadData(string[] records)
        {
            foreach (var rec in records)
            {
                _setLocomotives.Insert(DrawningObjectLocomotive.Create(rec) as T);
            }
        }


    }
}
